from typing import List, Optional, Tuple
import datetime
import base64
import json
import uuid
from .authentication import generate_csr, generate_key, Auth
from .types import (
    DataRoomDescription,
    DatasetDescription,
    UserCsrResponse,
    UserCsrRequest,
)
from .api import API, Endpoints
from .proto import (
    AuthenticationMethod, TrustedPki, DataRoom, DataRoomStatus, ConfigurationModification
)
from .config import (
    DECENTRIQ_CLIENT_ID,
    DECENTRIQ_API_PLATFORM_HOST, DECENTRIQ_API_PLATFORM_PORT, DECENTRIQ_API_PLATFORM_USE_TLS
)

__all__ = [
    "ClientPlatformFeatures", "SessionPlatformFeatures"
]


def platform_hash_from_str(s: str) -> bytes:
    """
    The reverse operation of `platform_hash_to_str`.
    """
    list_u8 = [int(x) for x in base64.b64decode(s).decode('ascii').split(',')]
    return bytes(list_u8)


def platform_hash_to_str(bs: bytes) -> str:
    """
    Transform a bytestring received from the
    The platform stores hashes (both files and data rooms) in the following form:
    Given a series of bytes with values 1 2 3, it will store the ascii codes of the
    string '1,2,3' (including the commas), so we get the byte array 49 44 50 44 51.
    This is then base64 encoded and sent to the backend where it is base64 decoded.
    """
    return base64.b64encode(','.join([str(b) for b in bs]).encode('ascii')).decode('ascii')


_GET_DATASET_LINKS_QUERY = """
query getDatasetLinks($filter: DatasetLinkFilter!) {
    datasetLinks(filter: $filter) {
        nodes {
            datasetLinkUuid: datasetLinkId
            computeNode {
                computeNodeUuid: computeNodeId
                nodeName
                dataRoom {
                    dataRoomUuid: dataRoomId
                    dataRoomId: dataRoomHashEncoded
                }
            }
            dataset {
                name
                datasetId: datasetHashEncoded
            }
        }
    }
}
"""


class PlatformApi:
    """
    Class to interact with the GraphQL endpoint of the Decentriq platform.

    This class enables the integration of the Python SDK with the browser-based
    Decentriq platform UI by providing the necessary CRUD functions.
    """

    user_email: str
    _http_api: API

    def __init__(self, user_email: str, http_api: API):
        self.user_email = user_email
        self._http_api = http_api

    def get_data_room_descriptions(self) -> List[DataRoomDescription]:
        data = self._post_graphql(
            """
            query getDataRooms($filter: DataRoomFilter) {
                dataRooms(filter: $filter, orderBy: NAME_ASC) {
                    nodes {
                        dataRoomId: dataRoomHashEncoded
                        name
                        description
                        mrenclave
                        ownerEmail
                        creationDate: createdAt
                        state {
                            status
                        }
                    }
                }
            }
            """
        )
        return self._parse_get_data_rooms_response(data)

    def update_data_room_status(
            self,
            data_room_hash: str,
            status # type: DataRoomStatus.V
    ):
        data_room = self.get_data_room_by_hash(data_room_hash)
        if not data_room:
            raise Exception(f"Unable to find data room with hash '{data_room_hash}'")
        data_room_uuid = data_room["dataRoomUuid"]
        current_datetime_str = datetime.datetime.now().isoformat()
        self._post_graphql(
            """
            mutation upsertState($input: UpsertStateInput!) {
                upsertState(input: $input) {
                    state {
                        id
                        status
                        statusUpdatedAt
                        updatedAt
                        updatedByEmail
                    }
                }
            }
            """,
            {
                "input": {
                    "clientMutationId": str(uuid.uuid4()),
                    "state": {
                        "dataRoomId": data_room_uuid,
                        "status": DataRoomStatus.Name(status).upper(),
                        "statusUpdatedAt": current_datetime_str,
                    }
                }
            }
        )

    def get_dataset_link(
            self,
            data_room_id: str,
            leaf_id: str,
    ) -> Optional[dict]:
        data_room = self.get_data_room_by_hash(data_room_id)
        if not data_room:
            raise Exception(f"Unable to get data room for hash '{data_room_id}'")
        data_room_uuid = data_room["dataRoomUuid"]
        compute_node = self._get_compute_node(data_room_uuid, leaf_id)
        if compute_node:
            compute_node_uuid = compute_node["computeNodeUuid"]
            return self._get_dataset_link(compute_node_uuid)
        else:
            raise Exception(
                f"Unable to find leaf with name '{leaf_id}' for data room '{data_room_id}'"
            )

    def get_dataset_links_for_manifest_hash(
            self,
            manifest_hash: str,
    ) -> Optional[List[str]]:
        links = self._get_dataset_links_for_manifest_hash(manifest_hash)
        return [link["datasetLinkUuid"] for link in links]

    def _get_dataset_links_for_manifest_hash(
            self,
            manifest_hash: str,
    ) -> List[dict]:
        data = self._post_graphql(
            _GET_DATASET_LINKS_QUERY,
            {
                "filter": {
                    "datasetHashEncoded": {"equalTo": manifest_hash},
                }
            }
        )
        return data["datasetLinks"]["nodes"]

    def _get_dataset_link(self, compute_node_uuid: str) -> Optional[dict]:
        data = self._post_graphql(
            _GET_DATASET_LINKS_QUERY,
            {
                "filter": {
                    "computeNode": {
                        "computeNodeId": {"equalTo": compute_node_uuid}
                    }
                }
            }
        )

        nodes = data["datasetLinks"]["nodes"]
        if nodes:
            return nodes[0]
        else:
            return None

    def get_data_rooms_with_published_dataset(self, manifest_hash) -> List[str]:
        links = self._get_dataset_links_for_manifest_hash(manifest_hash)
        data_room_ids = []
        for link in links:
            if "computeNode" in link and "dataRoom" in link["computeNode"]:
                data_room_id = link["computeNode"]["dataRoom"].get("dataRoomId")
                if data_room_id:
                    data_room_ids.append(data_room_id)
        return data_room_ids

    def delete_dataset_link(
            self,
            data_room_id: str,
            leaf_id: str,
    ) -> Optional[dict]:
        dataset_link = self.get_dataset_link(data_room_id, leaf_id)
        if not dataset_link:
            raise Exception(
                f"Unable to find a dataset link for data room '{data_room_id}'" +
                f" and data node '{leaf_id}'"
            )
        else:
            self._post_graphql(
                """
                mutation deleteDatasetLink($input: DeleteDatasetLinkInput!) {
                    deleteDatasetLink(input: $input) {
                        clientMutationId
                    }
                }
                """,
                {
                    "input": {
                        "clientMutationId": str(uuid.uuid4()),
                        "datasetLinkId": dataset_link["datasetLinkUuid"]
                    }
                }
            )

    def delete_dataset_links_for_manifest_hash(
            self,
            manifest_hash: str,
    ) -> Optional[dict]:
        uuids = self.get_dataset_links_for_manifest_hash(manifest_hash)
        if uuids:
            for link_id in uuids:
                self._post_graphql(
                    """
                    mutation deleteDatasetLink($input: DeleteDatasetLinkInput!) {
                        deleteDatasetLink(input: $input) {
                            clientMutationId
                        }
                    }
                    """,
                    {
                        "input": {
                            "clientMutationId": str(uuid.uuid4()),
                            "datasetLinkId": link_id
                        }
                    }
                )

    def _delete_data_room(self, data_room_uuid: str) -> str:
        data = self._post_graphql(
            """
            mutation deleteDataRoom($input: DeleteDataRoomByIdInput!) {
                deleteDataRoomById(input: $input) {
                    clientMutationId
                    dataRoom {
                        id
                    }

                }
            }
            """,
            {
                "input": {
                    "clientMutationId": str(uuid.uuid4()),
                    "id": data_room_uuid,
                }
            }
        )
        deleted_id = data.get("deleteDataRoomById", {}).get("dataRoom", {}).get("id")
        if deleted_id is None:
            raise Exception(
                "Received malformed response when trying to delete DCR in backend"
            )
        else:
            return data_room_uuid

    def publish_data_room(
            self,
            data_room_definition: Tuple[DataRoom, List[ConfigurationModification]],
            data_room_hash: str,
            attestation_specification_hash: str,
            additional_fields: dict = {},
    ):
        data_room, conf_modifications = data_room_definition
        owner_email = data_room.ownerEmail

        participant_emails = [
            op.add.element.userPermission.email
            for op in conf_modifications
            if op.HasField("add") and op.add.element.HasField("userPermission")
        ]

        if owner_email in participant_emails:
            participant_emails.remove(owner_email)

        user_permissions_input = []
        for participant_email in participant_emails:
            user_permissions_input.append({
                "email": participant_email,
            })

        data = self._post_graphql(
            """
            mutation createDataRoom($input: CreateDataRoomInput!) {
                createDataRoom(input: $input) {
                    dataRoom {
                        dataRoomUuid: dataRoomId
                        dataRoomHashEncoded
                        lock {
                            isLocked
                        }
                    }
                }
            }
            """,
            {
                "input": {
                    "clientMutationId": str(uuid.uuid4()),
                    "dataRoom": {
                        "dataRoomHash": platform_hash_to_str(bytes.fromhex(data_room_hash)),
                        "dataRoomHashEncoded": data_room_hash,
                        "name": data_room.name,
                        "description": data_room.description,
                        "mrenclave": attestation_specification_hash,
                        "source": "PYTHON",
                        "ownerEmail": data_room.ownerEmail,
                        "userPermissions": {
                            "create": user_permissions_input
                        },
                        **additional_fields,
                        "lock": {
                            "create": {
                                "isLocked": True
                            }
                        }
                    }
                }
            }
        )

        data_room_uuid = data.get("createDataRoom", {}).get("dataRoom", {}).get("dataRoomUuid")
        if data_room_uuid is None:
            raise Exception(
                "Received malformed response when trying to create DCR in backend"
            )

    def _parse_get_data_rooms_response(self, data) -> List[DataRoomDescription]:
        remove_keys = set(["state"])
        def payload_to_dcr_description(d):
            # A DCR without a state should be displayed as a DCR with "Active" status
            # Uppercase status such as "STOPPED" should be displayed as "Stopped" to match
            # the proto version.
            if d.get("state") and d["state"].get("status"):
                status = d["state"]["status"].capitalize()
            else:
                status = "Active"
            d_cleaned = {k: v for k, v in d.items() if k not in remove_keys}
            return DataRoomDescription(status=status, **d_cleaned)

        dcr_dicts = data.get("dataRooms", {}).get("nodes", [])
        dcr_descriptions = [payload_to_dcr_description(d) for d in dcr_dicts]

        # Remove non-published DCRs from list
        return [desc for desc in dcr_descriptions if desc["dataRoomId"]]

    def get_data_room_by_hash(self, data_room_hash: str) -> Optional[dict]:
        data = self._post_graphql(
            """
            query getDataRoomByHash($dataRoomHashEncoded: String!) {
                dataRooms(condition: {dataRoomHashEncoded: $dataRoomHashEncoded}) {
                    nodes {
                        dataRoomUuid: dataRoomId
                        source
                    }
                }
            }
            """,
            { "dataRoomHashEncoded": data_room_hash }
        )
        entries = data.get("dataRooms", {}).get("nodes", [])

        if len(entries) > 1:
            raise Exception("Cannot have multiple DCRs with the same hashcode")
        elif len(entries) == 0:
            return None
        else:
            return entries[0]

    def get_datasets_of_user(self) -> List[DatasetDescription]:
        data = self._post_graphql(
            """
            query getDatasets {
                datasets {
                    nodes {
                        datasetId: datasetHashEncoded
                        name
                        description
                        ownerEmail
                        creationDate: createdAt
                        datasetMeta {
                            description
                        }
                    }
                }
            }
            """
        )
        nodes = data.get("datasets", {}).get("nodes", [])
        datasets = []
        for node in nodes:
            meta_info = node.get("datasetMeta")
            if meta_info:
                description = meta_info.get("description")
            else:
                description = None
            datasets.append(
                DatasetDescription(
                    datasetId=node["datasetId"],
                    name=node["name"],
                    description=description,
                    ownerEmail=node["ownerEmail"],
                    creationDate=node["creationDate"]
                )
            )
        return datasets

    def save_dataset_metadata(
            self,
            manifest_hash: str,
            file_name: str,
            description: str,
            owner_email: str
    ):
        self._post_graphql(
            """
            mutation createDatasetMeta($input: CreateDatasetMetaInput!) {
                createDatasetMeta(input: $input) {
                    datasetMeta {
                        datasetHashEncoded
                        name
                        description
                    }
                }
            }
            """,
            {
                "input": {
                    "clientMutationId": str(uuid.uuid4()),
                    "datasetMeta": {
                        "datasetHash": platform_hash_to_str(bytes.fromhex(manifest_hash)),
                        "name": file_name,
                        "description": description,
                        "ownerEmail": owner_email,
                    }
                }
            }
        )

    def delete_dataset_metadata(self, manifest_hash: str):
        self._post_graphql(
            """
            mutation deleteDatasetMeta($datasetHashEncoded: String!) {
                deleteDatasetMetaByDatasetHashEncoded(input: {datasetHashEncoded: $datasetHashEncoded}) {
                    datasetMeta {
                        id
                    }
                }
            }
            """,
            {
                "datasetHashEncoded":  manifest_hash
            }
        )

    def create_dataset_link(
            self,
            data_room_id: str,
            manifest_hash: str,
            leaf_id: str
    ):
        data_room = self.get_data_room_by_hash(data_room_id)
        if not data_room:
            raise Exception(f"Unable to get data room id for hash {data_room_id}")
        else:
            if data_room["source"] == "WEB":
                data_room_uuid = data_room["dataRoomUuid"]
                compute_node = self._get_compute_node(data_room_uuid, leaf_id)
                if compute_node:
                    compute_node_uuid = compute_node["computeNodeUuid"]
                    # Can link to both compute nodes (type BRANCH) and leaf nodes (type LEAF).
                    # For SQL nodes we need to link to the verifier compute node.
                    if compute_node_uuid:
                        existing_link = self._get_dataset_link(compute_node_uuid)
                        if existing_link:
                            dataset_hash = existing_link["dataset"]["datasetId"]
                            raise Exception(
                                "The following dataset has already been published for this node." +
                                " Please unpublish this dataset first." +
                                f" Dataset: '{dataset_hash}'"
                            )
                        self._post_graphql(
                            """
                            mutation createDatasetLink($input: CreateDatasetLinkInput!) {
                                createDatasetLink(input: $input) {
                                    clientMutationId
                                    datasetLink {
                                        datasetHashEncoded
                                    }
                                }
                            }
                            """,
                            {
                                "input": {
                                    "clientMutationId": str(uuid.uuid4()),
                                    "datasetLink": {
                                        "datasetHash": platform_hash_to_str(
                                            bytes.fromhex(manifest_hash)
                                        ),
                                        "computeNodeId": compute_node_uuid,
                                    }
                                }
                            }
                        )
                else:
                    raise Exception(
                        f"Unable to find leaf with name '{leaf_id}' for data room '{data_room_id}'"
                    )
            else:
                pass

    def get_datasets_by_ids(
            self,
            manifest_hashes: List[str]
    ) -> List[DatasetDescription]:
        """
        Returns the a list of datasets with the given ids.
        """
        data = self._post_graphql(
            """
            query getDatasets($filter: DatasetMetaFilter!) {
                datasetMetas(filter: $filter) {
                    nodes {
                        datasetId: datasetHashEncoded
                        name
                        description
                        ownerEmail
                        creationDate: createdAt
                    }
                }
            }
            """,
            {
                "filter": {
                    "datasetHashEncoded": {
                        "in": manifest_hashes
                    }
                }
            }
        )
        nodes = data.get("datasetMetas", {}).get("nodes", [])
        return [DatasetDescription(**node) for node in nodes]

    def _get_compute_node(self, data_room_uuid: str, leaf_id: str) -> Optional[dict]:
        data = self._post_graphql(
            """
            query getComputeNodes($filter: ComputeNodeFilter!) {
                computeNodes(filter: $filter) {
                    nodes {
                        computeNodeUuid: computeNodeId
                        nodeName
                        computeNodeType
                    }
                }
            }
            """,
            {
                "filter": {
                    "dataRoomId": { "equalTo": data_room_uuid },
                    "nodeName": { "equalTo": leaf_id },
                }
            }
        )
        nodes = data["computeNodes"]["nodes"]
        if nodes and len(nodes) == 1:
            return nodes[0]
        else:
            return None

    def _post_graphql(self, query: str, variables={}) -> dict:
        request_payload = { "query": query }
        if variables:
            request_payload["variables"] = variables
        response = self._http_api.post(
            "",
            json.dumps(request_payload),
            {"Content-type": "application/json"}
        )
        payload = response.json()
        if "errors" in payload:
            error_messages = []
            for message in payload["errors"]:
                decoded_message =\
                    base64.b64decode(message["message"]).decode('utf-8')
                error_messages.append(decoded_message)
            raise Exception(",".join(error_messages))
        elif "data" in payload:
            return payload["data"]
        else:
            raise Exception("Malformed GraphQL response: no 'data' or 'errors' key")


def create_platform_api(
    api_token: str,
    user_email: str,
    client_id: str = DECENTRIQ_CLIENT_ID,
    api_host: str = DECENTRIQ_API_PLATFORM_HOST,
    api_port: int = DECENTRIQ_API_PLATFORM_PORT,
    api_use_tls: bool = DECENTRIQ_API_PLATFORM_USE_TLS
):
    http_api = API(
        api_token,
        client_id,
        api_host,
        api_port,
        api_prefix="/api/decentriq-platform/graphql",
        use_tls=api_use_tls,
        additional_auth_headers={
            "Authorization-User-Email": user_email,
        }
    )
    return PlatformApi(user_email, http_api)


class ClientPlatformFeatures:
    """
    Provider of a list of methods and properties mirroring what is offered by the Decentriq
    web platform.
    """
    _http_api: API
    _platform_api: PlatformApi

    def __init__(
        self,
        api_token: str,
        user_email: str,
        http_api: API,
        client_id: str = DECENTRIQ_CLIENT_ID,
        api_host: str = DECENTRIQ_API_PLATFORM_HOST,
        api_port: int = DECENTRIQ_API_PLATFORM_PORT,
        api_use_tls: bool = DECENTRIQ_API_PLATFORM_USE_TLS,
    ):
        """
        Creating objects of this class directly is not necessary as an object of this class is
        provided as part of each `Client` object.
        """
        self._http_api = http_api
        self._platform_api = create_platform_api(api_token, user_email, client_id, api_host, api_port, api_use_tls)
        self.user_email = user_email

    @property
    def decentriq_ca_root_certificate(self) -> bytes:
        """
        Returns the root certificate used by the Decentriq identity provider.
        Note that when using this certificate in any authentication scheme, you trust Decentriq as an identity provider!
        """
        url = Endpoints.SYSTEM_CERTIFICATE_AUTHORITY
        response = self._http_api.get(url).json()
        return response["rootCertificate"].encode("utf-8")

    @property
    def decentriq_pki_authentication(self) -> AuthenticationMethod:
        """
        The authentication method that uses the Decentriq root certificate to authenticate
        users.

        This method should be specified when building a data room in case you want to interact
        with the that data room either via the web interface or with sessions created using
        `create_auth_using_decentriq_pki`.
        Note that when using this authentication method you trust Decentriq as an identity provider!

        You can also create an `AuthenticationMethod` object directly and supply your own root certificate,
        with which to authenticate users connecting to your data room.
        In this case you will also need to issue corresponding user certificates and create your
        own custom `decentriq_platform.authentication.Auth` objects.
        """
        root_pki = self.decentriq_ca_root_certificate
        return AuthenticationMethod(
            trustedPki=TrustedPki(rootCertificatePem=root_pki)
        )

    def create_auth_using_decentriq_pki(self, email: str = None) -> Auth:
        """
        Creates a `decentriq_platform.authentication.Auth` object which can be
        attached to a `decentriq_platform.session.Session`.
        Sessions created using such an `Auth` object will commonly be used with
        data rooms that have been configured to use the `decentriq_pki_authentication`
        authentication method.
        """
        email = email if email is not None else self.user_email
        keypair = generate_key()
        csr = generate_csr(email, keypair)
        url = Endpoints.USER_CERTIFICATE.replace(":userId", email)
        csr_req = UserCsrRequest(csrPem=csr.decode("utf-8"))
        resp: UserCsrResponse = self._http_api.post(url, req_body=json.dumps(csr_req)).json()
        cert_chain_pem = resp["certChainPem"].encode("utf-8")
        auth = Auth(cert_chain_pem, keypair, email)
        return auth

    def get_data_room_descriptions(self) -> List[DataRoomDescription]:
        """
        Returns the a list of descriptions of all the data rooms a user created
        or participates in.
        """
        return self._platform_api.get_data_room_descriptions()

    def get_available_datasets(self) -> List[DatasetDescription]:
        """
        Returns the a list of datasets that the current user uploaded, regardless
        of whether they have already been connected to a data room or not.
        """
        return self._platform_api.get_datasets_of_user()


class SessionPlatformFeatures:
    """
    A provider for methods that mirror functionality known from the Decentriq
    web platform.

    This class is similar to `ClientPlatformFeatures` but methods provided on
    this class require communication with an enclave.
    """

    _platform_api: PlatformApi

    def __init__(self, session, api: PlatformApi):
        """
        Instances of this class should not be created directly but rather be
        accessed via the `Session.platform` field.
        """
        self._session = session
        self._platform_api = api

    def get_published_datasets(
            self,
            data_room_id: str
    ) -> List[DatasetDescription]:
        """
        Get a list of all the datasets that were published for a given data room.
        """
        response = self._session.retrieve_published_datasets(data_room_id)
        manifest_hashes =\
            [dataset.datasetHash.hex() for dataset in response.publishedDatasets]
        return self._platform_api.get_datasets_by_ids(manifest_hashes)



import os

DECENTRIQ_CLIENT_ID = os.getenv("DECENTRIQ_CLIENT_ID", "MHyVW112w7Ql95G96fn9rnLWkYuOLmdk")

DECENTRIQ_HOST = os.getenv("DECENTRIQ_HOST", "api.decentriq.com")
DECENTRIQ_PORT = int(os.getenv("DECENTRIQ_PORT", "443"))
DECENTRIQ_USE_TLS = True if os.getenv("DECENTRIQ_USE_TLS", "true") == "true" else False

DECENTRIQ_API_CORE_HOST = os.getenv("DECENTRIQ_API_CORE_HOST", None)

if "DECENTRIQ_API_CORE_PORT" in os.environ:
    DECENTRIQ_API_CORE_PORT = int(os.getenv("DECENTRIQ_API_CORE_PORT"))
else:
    DECENTRIQ_API_CORE_PORT = None

if "DECENTRIQ_API_CORE_USE_TLS" in os.environ:
    DECENTRIQ_API_CORE_USE_TLS = True if os.getenv("DECENTRIQ_API_CORE_USE_TLS") == "true" else False
else:
    DECENTRIQ_API_CORE_USE_TLS = None

DECENTRIQ_API_PLATFORM_HOST = os.getenv("DECENTRIQ_API_PLATFORM_HOST", None)

if "DECENTRIQ_API_PLATFORM_PORT" in os.environ:
    DECENTRIQ_API_PLATFORM_PORT = int(os.getenv("DECENTRIQ_API_PLATFORM_PORT"))
else:
    DECENTRIQ_API_PLATFORM_PORT = None

if "DECENTRIQ_API_PLATFORM_USE_TLS" in os.environ:
    DECENTRIQ_API_PLATFORM_USE_TLS = True if os.getenv("DECENTRIQ_API_PLATFORM_USE_TLS") == "true" else False
else:
    DECENTRIQ_API_PLATFORM_USE_TLS = None



"""
.. include:: ../../decentriq_platform_docs/gcg_getting_started.md
---
"""
__docformat__ = "restructuredtext"

from .client import Client, create_client, Session
from .platform import ClientPlatformFeatures, SessionPlatformFeatures
from .builders import (
    DataRoomBuilder,
    DataRoomCommitBuilder,
    GovernanceProtocol
)
from .compute import Noop, StaticContent
from .permission import Permissions
from .storage import Key
from .attestation import enclave_specifications, EnclaveSpecifications


__all__ = [
    "create_client",
    "Client",
    "Session",
    "DataRoomBuilder",
    "DataRoomCommitBuilder",
    "Permissions",
    "GovernanceProtocol",
    "enclave_specifications",
    "EnclaveSpecifications",
    "Key",
    "StaticContent",
    "Noop",
    "sql",
    "container",
    "storage",
    "attestation",
    "types",
    "authentication",
    "platform",
    "session",
    "node",
]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import attestation_pb2
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class ComputeNodeFormat(_ComputeNodeFormat, metaclass=_ComputeNodeFormatEnumTypeWrapper):
    pass
class _ComputeNodeFormat:
    V = typing.NewType('V', builtins.int)
class _ComputeNodeFormatEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ComputeNodeFormat.V], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    RAW = ComputeNodeFormat.V(0)
    ZIP = ComputeNodeFormat.V(1)

RAW = ComputeNodeFormat.V(0)
ZIP = ComputeNodeFormat.V(1)
global___ComputeNodeFormat = ComputeNodeFormat


class DataRoom(google.protobuf.message.Message):
    """/ To create a dataroom the user need to specify the `GovernanceProtocol`
    / and the list of `ConfigurationModification`.
    / The `GovernanceProtocol` defines how new modifications can be applied to an existing data room.
    / The list of `ConfigurationModification`s (abbreviated by "Modification" in the diagram below)
    / defined the structure of the data room itself (the compute nodes,
    / the permissions, etc...), this list makes up the first `ConfigurationCommit` for the dataroom.
    /
    / ```diagram
    /  ┌──────────────────────┐                               
    /  │CreateDataRoomRequest │           ┌──────────────────┐
    /  │ ┌──────────────────┐ │           │ DataRoomContext  │
    /  │ │GovernanceProtocol│ │           │ ┌──────────────┐ │
    /  │ └──────────────────┘ │           │ │  Leaf nodes  │ │
    /  │ ┌─────────────┐      ├──────────▶│ │ manifest map │ │
    /  │ │ ┌───────────┴─┐    │           │ └──────────────┘ │
    /  │ └─┤┌────────────┴──┐ │           │ ┌──────────────┐ │
    /  │   └┤ Modification  │ │           │ │CommitsHistory│ │
    /  │    └───────────────┘ │           │ └──────────────┘ │
    /  └──────────────────────┘           └──────────────────┘
    / ```
    /
    / A dataroom has a corresponding `DataRoomContext` which includes a map of the
    / currently published datasets (and corresponding encryption key) and the commits history
    / which is a list of the `ConfigurationCommit` that make up the data room
    /  
    / ```diagram
    /  ┌──────────────────────────────┐                                        
    /  │        CommitsHistory        │                                        
    /  │┌────────────────────┐        │        ┌────┬─────────────────────────┐
    /  ││ ┌──────────────────┴─┐      │     ┌──│ Id │ConfigurationCommit #1   │
    /  ││ │  ┌─────────────────┴──┐   │     │  └────┴─────────────────────────┘
    /  ││ │  │ ┌──────────────────┴─┐ │     │  ┌────┬─────────────────────────┐
    /  ││ │  │ │ConfigurationCommit │ │     ├──│ Id │ConfigurationCommit #2   │
    /  ││ │  │ │┌──────────────────┐│ │     │  └────┴─────────────────────────┘
    /  ││ │  │ ││    DataRoomId    ││ │     │  ┌────┬─────────────────────────┐
    /  ││ │  │ │└──────────────────┘│ │     ├──│ Id │ConfigurationCommit #3   │
    /  ││ │  │ │┌──────────────────┐│ │     │  └────┴─────────────────────────┘
    /  ││ │  │ ││   DataRoomPin    ││ │     │  ┌────┬─────────────────────────┐
    /  ││ │  │ │└──────────────────┘│ │     ├──│ Id │ConfigurationCommit #4   │
    /  ││ │  │ │┌─────────────┐     │ │     │  └────┴─────────────────────────┘
    /  ││ │  │ ││ ┌───────────┴─┐   │ │     │                                  
    /  │└─┤  │ │└─┤┌────────────┴──┐│ │     │                                  
    /  │  └──┤ │  └┤ Modification  ││ │     │    ┌──────────────────────────┐  
    /  │     └─┤   └───────────────┘│ │     └───▶│       DataRoomPin        │  
    /  │       └────────────────────┘ │          └──────────────────────────┘  
    /  └──────────────────────────────┘
    / ```
    /
    / Each `ConfigurationCommit` is identified by the data room which it refers to and the
    / pin of the data room itself. A commit can only be merged if the `pin` corresponds to the
    / current configuration history state of the data room
    /
    / ```diagram
    /  ┌───────────────────────────────┐           ┌────────────────┐
    /  │MergeConfigurationCommitRequest│           │ CommitsHistory │
    /  │ ┌───────────────────────────┐ │           └────────────────┘
    /  │ │  ConfigurationCommit #5   │ │              ┌────┐         
    /  │ └───────────────────────────┘ │              │ ┌──┴─┐       
    /  │ ┌─────────────────────┐       ├──────────▶   └─┤ ┌──┴─┐     
    /  │ │  ┌──────────────────┴──┐    │                └─┤ ┌──┴─┐   
    /  │ └──┤  ┌──────────────────┴──┐ │                  └─┤ ┌──┴─┐ 
    /  │    └──┤   MergeSignature    │ │                    └─┤ 5  │ 
    /  │       └─────────────────────┘ │                      └────┘ 
    /  └───────────────────────────────┘                             
    / ```
    /
    / When a user collects the approval signatures required to merge a commit, this can
    / be added to the commits history.
    / The list of approvers depends both on the commit and the `GovernanceProtocol` that
    / the data room is configured with:
    / - StaticDataRoomPolicy: the data room is static, there isn't an approver that
    /    can authorize the merge
    / - AffectedDataOwnersApprovePolicy: the approvers for a commit are the user which have
    /     a CrudPermisison on the leaf node that the new nodes may use
    /
    / ```diagram
    /                                           ┌────────────────────┐                         
    /                                           │   UserPermission   │                         
    /  ┌ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┐      │ ┌───────────────┐  │                         
    /            New Commit Node                │ │user1@email.com│──┼──┐                      
    /  │          ┌───┬───────┐          │      │ └───────────────┘  │  │                      
    /           ┌▶│0x4│Python │◀─┐              │ ┌──────────────┐   │  │                      
    /  │        │ └───┴───────┘  │       │      │ │ ┌────────────┴─┐ │  │                      
    /   ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─│─ ─ ─ ─       │ └─┤CrudPermission│ │  │     ┌───────────────┐
    /  ┌ ─ ─ ─ ─│─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┐      │   │   id: 0x1    │ │  └────▶│   Required    │
    /     ┌───┬─┴─────┐          │              │   └──────────────┘ │        │   Approvers   │
    /  │  │0x3│  Sql  │          │       │      └────────────────────┘        ├───────────────┤
    /     └───┴───────┘          │              ┌────────────────────┐  ┌────▶│user1@email.com│
    /  │        ▲                │       │      │   UserPermission   │  │     │               │
    /     ┌───┬─┴─────┐   ┌───┬──┴────┐         │ ┌───────────────┐  │  │     │user2@email.com│
    /  │  │0x1│Leaf 1 │   │0x2│Leaf 2 │  │      │ │user2@email.com│──┼──┘     └───────────────┘
    /     └───┴───────┘   └───┴───────┘         │ └───────────────┘  │                         
    /  │                                 │      │ ┌──────────────┐   │                         
    /      Old data room configuration          │ │ ┌────────────┴─┐ │                         
    /  └ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┘      │ └─┤CrudPermission│ │                         
    /                                           │   │   id: 0x2    │ │                         
    /                                           │   └──────────────┘ │                         
    /                                           └────────────────────┘
    / ```
    /
    / In the example above `user3@email.com` has created a new commit which adds node `0x4` to the
    / data room. By walking the computation graph the node depends on node `0x2` directly and node
    / `0x1` indirectly. To authorize the merge both owners of these leaf nodes must provide an approval
    / signature. The ownership of the nodes is determined by checking who has `CrudPermission` on the
    / leaf nodes.
    /  
    / Configuration commits don't have to be merged into a data room configuration
    / history, but can also be used to execute computations on top of data the user
    / already has access to. This is determined by checking if a user has ExecuteComputePermission
    / on the node which the new nodes depend on
    /
    / ```diagram
    /  ┌ ─ ─ ─ ─ ─ ─ ─ ─ ┐                          
    /    New Commit Node                            
    /  │  ┌───┬───────┐  │                          
    /     │0x3│Python │       ┌────────────────────┐
    /  │  └───┴───────┘  │    │   UserPermission   │
    /   ─ ─ ─ ─ ▲ ─ ─ ─ ─     │ ┌───────────────┐  │
    /  ┌ ─ ─ ─ ─│─ ─ ─ ─ ┐    │ │user2@email.com│  │
    /     ┌───┬─┴─────┐       │ └───────────────┘  │
    /  │  │0x2│  Sql  │  │    │ ┌──────────────┐   │
    /     └───┴───────┘       │ │ ┌────────────┴─┐ │
    /  │        ▲        │    │ └─┤ExecuteCompute│ │
    /     ┌───┬─┴─────┐       │   │   id: 0x2    │ │
    /  │  │0x1│Leaf 1 │  │    │   └──────────────┘ │
    /     └───┴───────┘       └────────────────────┘
    /  │  Old data room  │                          
    /     configuration                             
    /  └ ─ ─ ─ ─ ─ ─ ─ ─ ┘
    / ```
    /
    / In the example above `user2@email.com` has created a new commit which adds node `0x3` to the
    / data room. The node has a dependency on `0x2`, for which the user has `ExecuteComputePermission`
    / which means that the user can execute the node `0x3`

    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    OWNEREMAIL_FIELD_NUMBER: builtins.int
    GOVERNANCEPROTOCOL_FIELD_NUMBER: builtins.int
    id: typing.Text = ...
    name: typing.Text = ...
    description: typing.Text = ...
    ownerEmail: typing.Text = ...
    @property
    def governanceProtocol(self) -> global___GovernanceProtocol: ...
    def __init__(self,
        *,
        id : typing.Text = ...,
        name : typing.Text = ...,
        description : typing.Text = ...,
        ownerEmail : typing.Text = ...,
        governanceProtocol : typing.Optional[global___GovernanceProtocol] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"governanceProtocol",b"governanceProtocol"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"description",b"description",u"governanceProtocol",b"governanceProtocol",u"id",b"id",u"name",b"name",u"ownerEmail",b"ownerEmail"]) -> None: ...
global___DataRoom = DataRoom

class GovernanceProtocol(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    STATICDATAROOMPOLICY_FIELD_NUMBER: builtins.int
    AFFECTEDDATAOWNERSAPPROVEPOLICY_FIELD_NUMBER: builtins.int
    @property
    def staticDataRoomPolicy(self) -> global___StaticDataRoomPolicy: ...
    @property
    def affectedDataOwnersApprovePolicy(self) -> global___AffectedDataOwnersApprovePolicy: ...
    def __init__(self,
        *,
        staticDataRoomPolicy : typing.Optional[global___StaticDataRoomPolicy] = ...,
        affectedDataOwnersApprovePolicy : typing.Optional[global___AffectedDataOwnersApprovePolicy] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"affectedDataOwnersApprovePolicy",b"affectedDataOwnersApprovePolicy",u"policy",b"policy",u"staticDataRoomPolicy",b"staticDataRoomPolicy"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"affectedDataOwnersApprovePolicy",b"affectedDataOwnersApprovePolicy",u"policy",b"policy",u"staticDataRoomPolicy",b"staticDataRoomPolicy"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"policy",b"policy"]) -> typing.Optional[typing_extensions.Literal["staticDataRoomPolicy","affectedDataOwnersApprovePolicy"]]: ...
global___GovernanceProtocol = GovernanceProtocol

class StaticDataRoomPolicy(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___StaticDataRoomPolicy = StaticDataRoomPolicy

class AffectedDataOwnersApprovePolicy(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___AffectedDataOwnersApprovePolicy = AffectedDataOwnersApprovePolicy

class DataRoomConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    class ElementsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text = ...
        @property
        def value(self) -> global___ConfigurationElement: ...
        def __init__(self,
            *,
            key : typing.Text = ...,
            value : typing.Optional[global___ConfigurationElement] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"value",b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"key",b"key",u"value",b"value"]) -> None: ...

    ELEMENTS_FIELD_NUMBER: builtins.int
    @property
    def elements(self) -> google.protobuf.internal.containers.MessageMap[typing.Text, global___ConfigurationElement]: ...
    def __init__(self,
        *,
        elements : typing.Optional[typing.Mapping[typing.Text, global___ConfigurationElement]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"elements",b"elements"]) -> None: ...
global___DataRoomConfiguration = DataRoomConfiguration

class ConfigurationElement(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    COMPUTENODE_FIELD_NUMBER: builtins.int
    ATTESTATIONSPECIFICATION_FIELD_NUMBER: builtins.int
    USERPERMISSION_FIELD_NUMBER: builtins.int
    AUTHENTICATIONMETHOD_FIELD_NUMBER: builtins.int
    @property
    def computeNode(self) -> global___ComputeNode: ...
    @property
    def attestationSpecification(self) -> attestation_pb2.AttestationSpecification: ...
    @property
    def userPermission(self) -> global___UserPermission: ...
    @property
    def authenticationMethod(self) -> global___AuthenticationMethod: ...
    def __init__(self,
        *,
        computeNode : typing.Optional[global___ComputeNode] = ...,
        attestationSpecification : typing.Optional[attestation_pb2.AttestationSpecification] = ...,
        userPermission : typing.Optional[global___UserPermission] = ...,
        authenticationMethod : typing.Optional[global___AuthenticationMethod] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"attestationSpecification",b"attestationSpecification",u"authenticationMethod",b"authenticationMethod",u"computeNode",b"computeNode",u"element",b"element",u"userPermission",b"userPermission"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"attestationSpecification",b"attestationSpecification",u"authenticationMethod",b"authenticationMethod",u"computeNode",b"computeNode",u"element",b"element",u"userPermission",b"userPermission"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"element",b"element"]) -> typing.Optional[typing_extensions.Literal["computeNode","attestationSpecification","userPermission","authenticationMethod"]]: ...
global___ConfigurationElement = ConfigurationElement

class ConfigurationModification(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ADD_FIELD_NUMBER: builtins.int
    CHANGE_FIELD_NUMBER: builtins.int
    DELETE_FIELD_NUMBER: builtins.int
    @property
    def add(self) -> global___AddModification: ...
    @property
    def change(self) -> global___ChangeModification: ...
    @property
    def delete(self) -> global___DeleteModification: ...
    def __init__(self,
        *,
        add : typing.Optional[global___AddModification] = ...,
        change : typing.Optional[global___ChangeModification] = ...,
        delete : typing.Optional[global___DeleteModification] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"add",b"add",u"change",b"change",u"delete",b"delete",u"modification",b"modification"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"add",b"add",u"change",b"change",u"delete",b"delete",u"modification",b"modification"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"modification",b"modification"]) -> typing.Optional[typing_extensions.Literal["add","change","delete"]]: ...
global___ConfigurationModification = ConfigurationModification

class AddModification(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ID_FIELD_NUMBER: builtins.int
    ELEMENT_FIELD_NUMBER: builtins.int
    id: typing.Text = ...
    @property
    def element(self) -> global___ConfigurationElement: ...
    def __init__(self,
        *,
        id : typing.Text = ...,
        element : typing.Optional[global___ConfigurationElement] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"element",b"element"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"element",b"element",u"id",b"id"]) -> None: ...
global___AddModification = AddModification

class ChangeModification(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ID_FIELD_NUMBER: builtins.int
    ELEMENT_FIELD_NUMBER: builtins.int
    id: typing.Text = ...
    @property
    def element(self) -> global___ConfigurationElement: ...
    def __init__(self,
        *,
        id : typing.Text = ...,
        element : typing.Optional[global___ConfigurationElement] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"element",b"element"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"element",b"element",u"id",b"id"]) -> None: ...
global___ChangeModification = ChangeModification

class DeleteModification(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ID_FIELD_NUMBER: builtins.int
    id: typing.Text = ...
    def __init__(self,
        *,
        id : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"id",b"id"]) -> None: ...
global___DeleteModification = DeleteModification

class ConfigurationCommit(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATAROOMID_FIELD_NUMBER: builtins.int
    DATAROOMHISTORYPIN_FIELD_NUMBER: builtins.int
    MODIFICATIONS_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes = ...
    dataRoomHistoryPin: builtins.bytes = ...
    @property
    def modifications(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ConfigurationModification]: ...
    def __init__(self,
        *,
        dataRoomId : builtins.bytes = ...,
        dataRoomHistoryPin : builtins.bytes = ...,
        modifications : typing.Optional[typing.Iterable[global___ConfigurationModification]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dataRoomHistoryPin",b"dataRoomHistoryPin",u"dataRoomId",b"dataRoomId",u"modifications",b"modifications"]) -> None: ...
global___ConfigurationCommit = ConfigurationCommit

class ComputeNode(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    NODENAME_FIELD_NUMBER: builtins.int
    LEAF_FIELD_NUMBER: builtins.int
    BRANCH_FIELD_NUMBER: builtins.int
    nodeName: typing.Text = ...
    @property
    def leaf(self) -> global___ComputeNodeLeaf: ...
    @property
    def branch(self) -> global___ComputeNodeBranch: ...
    def __init__(self,
        *,
        nodeName : typing.Text = ...,
        leaf : typing.Optional[global___ComputeNodeLeaf] = ...,
        branch : typing.Optional[global___ComputeNodeBranch] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"branch",b"branch",u"leaf",b"leaf",u"node",b"node"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"branch",b"branch",u"leaf",b"leaf",u"node",b"node",u"nodeName",b"nodeName"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"node",b"node"]) -> typing.Optional[typing_extensions.Literal["leaf","branch"]]: ...
global___ComputeNode = ComputeNode

class ComputeNodeLeaf(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ISREQUIRED_FIELD_NUMBER: builtins.int
    isRequired: builtins.bool = ...
    def __init__(self,
        *,
        isRequired : builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"isRequired",b"isRequired"]) -> None: ...
global___ComputeNodeLeaf = ComputeNodeLeaf

class ComputeNodeProtocol(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VERSION_FIELD_NUMBER: builtins.int
    version: builtins.int = ...
    def __init__(self,
        *,
        version : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"version",b"version"]) -> None: ...
global___ComputeNodeProtocol = ComputeNodeProtocol

class ComputeNodeBranch(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CONFIG_FIELD_NUMBER: builtins.int
    DEPENDENCIES_FIELD_NUMBER: builtins.int
    OUTPUTFORMAT_FIELD_NUMBER: builtins.int
    PROTOCOL_FIELD_NUMBER: builtins.int
    ATTESTATIONSPECIFICATIONID_FIELD_NUMBER: builtins.int
    config: builtins.bytes = ...
    @property
    def dependencies(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    outputFormat: global___ComputeNodeFormat.V = ...
    @property
    def protocol(self) -> global___ComputeNodeProtocol: ...
    attestationSpecificationId: typing.Text = ...
    def __init__(self,
        *,
        config : builtins.bytes = ...,
        dependencies : typing.Optional[typing.Iterable[typing.Text]] = ...,
        outputFormat : global___ComputeNodeFormat.V = ...,
        protocol : typing.Optional[global___ComputeNodeProtocol] = ...,
        attestationSpecificationId : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"protocol",b"protocol"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"attestationSpecificationId",b"attestationSpecificationId",u"config",b"config",u"dependencies",b"dependencies",u"outputFormat",b"outputFormat",u"protocol",b"protocol"]) -> None: ...
global___ComputeNodeBranch = ComputeNodeBranch

class UserPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    EMAIL_FIELD_NUMBER: builtins.int
    PERMISSIONS_FIELD_NUMBER: builtins.int
    AUTHENTICATIONMETHODID_FIELD_NUMBER: builtins.int
    email: typing.Text = ...
    @property
    def permissions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Permission]: ...
    authenticationMethodId: typing.Text = ...
    def __init__(self,
        *,
        email : typing.Text = ...,
        permissions : typing.Optional[typing.Iterable[global___Permission]] = ...,
        authenticationMethodId : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"authenticationMethodId",b"authenticationMethodId",u"email",b"email",u"permissions",b"permissions"]) -> None: ...
global___UserPermission = UserPermission

class AuthenticationMethod(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    TRUSTEDPKI_FIELD_NUMBER: builtins.int
    @property
    def trustedPki(self) -> global___TrustedPki: ...
    def __init__(self,
        *,
        trustedPki : typing.Optional[global___TrustedPki] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"_trustedPki",b"_trustedPki",u"trustedPki",b"trustedPki"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"_trustedPki",b"_trustedPki",u"trustedPki",b"trustedPki"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"_trustedPki",b"_trustedPki"]) -> typing.Optional[typing_extensions.Literal["trustedPki"]]: ...
global___AuthenticationMethod = AuthenticationMethod

class TrustedPki(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ROOTCERTIFICATEPEM_FIELD_NUMBER: builtins.int
    rootCertificatePem: builtins.bytes = ...
    def __init__(self,
        *,
        rootCertificatePem : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"rootCertificatePem",b"rootCertificatePem"]) -> None: ...
global___TrustedPki = TrustedPki

class Permission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    EXECUTECOMPUTEPERMISSION_FIELD_NUMBER: builtins.int
    LEAFCRUDPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEAUDITLOGPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMSTATUSPERMISSION_FIELD_NUMBER: builtins.int
    UPDATEDATAROOMSTATUSPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEPUBLISHEDDATASETSPERMISSION_FIELD_NUMBER: builtins.int
    DRYRUNPERMISSION_FIELD_NUMBER: builtins.int
    GENERATEMERGESIGNATUREPERMISSION_FIELD_NUMBER: builtins.int
    EXECUTEDEVELOPMENTCOMPUTEPERMISSION_FIELD_NUMBER: builtins.int
    MERGECONFIGURATIONCOMMITPERMISSION_FIELD_NUMBER: builtins.int
    @property
    def executeComputePermission(self) -> global___ExecuteComputePermission: ...
    @property
    def leafCrudPermission(self) -> global___LeafCrudPermission: ...
    @property
    def retrieveDataRoomPermission(self) -> global___RetrieveDataRoomPermission: ...
    @property
    def retrieveAuditLogPermission(self) -> global___RetrieveAuditLogPermission: ...
    @property
    def retrieveDataRoomStatusPermission(self) -> global___RetrieveDataRoomStatusPermission: ...
    @property
    def updateDataRoomStatusPermission(self) -> global___UpdateDataRoomStatusPermission: ...
    @property
    def retrievePublishedDatasetsPermission(self) -> global___RetrievePublishedDatasetsPermission: ...
    @property
    def dryRunPermission(self) -> global___DryRunPermission: ...
    @property
    def generateMergeSignaturePermission(self) -> global___GenerateMergeSignaturePermission: ...
    @property
    def executeDevelopmentComputePermission(self) -> global___ExecuteDevelopmentComputePermission: ...
    @property
    def mergeConfigurationCommitPermission(self) -> global___MergeConfigurationCommitPermission: ...
    def __init__(self,
        *,
        executeComputePermission : typing.Optional[global___ExecuteComputePermission] = ...,
        leafCrudPermission : typing.Optional[global___LeafCrudPermission] = ...,
        retrieveDataRoomPermission : typing.Optional[global___RetrieveDataRoomPermission] = ...,
        retrieveAuditLogPermission : typing.Optional[global___RetrieveAuditLogPermission] = ...,
        retrieveDataRoomStatusPermission : typing.Optional[global___RetrieveDataRoomStatusPermission] = ...,
        updateDataRoomStatusPermission : typing.Optional[global___UpdateDataRoomStatusPermission] = ...,
        retrievePublishedDatasetsPermission : typing.Optional[global___RetrievePublishedDatasetsPermission] = ...,
        dryRunPermission : typing.Optional[global___DryRunPermission] = ...,
        generateMergeSignaturePermission : typing.Optional[global___GenerateMergeSignaturePermission] = ...,
        executeDevelopmentComputePermission : typing.Optional[global___ExecuteDevelopmentComputePermission] = ...,
        mergeConfigurationCommitPermission : typing.Optional[global___MergeConfigurationCommitPermission] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"dryRunPermission",b"dryRunPermission",u"executeComputePermission",b"executeComputePermission",u"executeDevelopmentComputePermission",b"executeDevelopmentComputePermission",u"generateMergeSignaturePermission",b"generateMergeSignaturePermission",u"leafCrudPermission",b"leafCrudPermission",u"mergeConfigurationCommitPermission",b"mergeConfigurationCommitPermission",u"permission",b"permission",u"retrieveAuditLogPermission",b"retrieveAuditLogPermission",u"retrieveDataRoomPermission",b"retrieveDataRoomPermission",u"retrieveDataRoomStatusPermission",b"retrieveDataRoomStatusPermission",u"retrievePublishedDatasetsPermission",b"retrievePublishedDatasetsPermission",u"updateDataRoomStatusPermission",b"updateDataRoomStatusPermission"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dryRunPermission",b"dryRunPermission",u"executeComputePermission",b"executeComputePermission",u"executeDevelopmentComputePermission",b"executeDevelopmentComputePermission",u"generateMergeSignaturePermission",b"generateMergeSignaturePermission",u"leafCrudPermission",b"leafCrudPermission",u"mergeConfigurationCommitPermission",b"mergeConfigurationCommitPermission",u"permission",b"permission",u"retrieveAuditLogPermission",b"retrieveAuditLogPermission",u"retrieveDataRoomPermission",b"retrieveDataRoomPermission",u"retrieveDataRoomStatusPermission",b"retrieveDataRoomStatusPermission",u"retrievePublishedDatasetsPermission",b"retrievePublishedDatasetsPermission",u"updateDataRoomStatusPermission",b"updateDataRoomStatusPermission"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"permission",b"permission"]) -> typing.Optional[typing_extensions.Literal["executeComputePermission","leafCrudPermission","retrieveDataRoomPermission","retrieveAuditLogPermission","retrieveDataRoomStatusPermission","updateDataRoomStatusPermission","retrievePublishedDatasetsPermission","dryRunPermission","generateMergeSignaturePermission","executeDevelopmentComputePermission","mergeConfigurationCommitPermission"]]: ...
global___Permission = Permission

class ExecuteComputePermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    COMPUTENODENAME_FIELD_NUMBER: builtins.int
    computeNodeName: typing.Text = ...
    def __init__(self,
        *,
        computeNodeName : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"computeNodeName",b"computeNodeName"]) -> None: ...
global___ExecuteComputePermission = ExecuteComputePermission

class LeafCrudPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    LEAFNODENAME_FIELD_NUMBER: builtins.int
    leafNodeName: typing.Text = ...
    def __init__(self,
        *,
        leafNodeName : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"leafNodeName",b"leafNodeName"]) -> None: ...
global___LeafCrudPermission = LeafCrudPermission

class RetrieveDataRoomPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___RetrieveDataRoomPermission = RetrieveDataRoomPermission

class RetrieveAuditLogPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___RetrieveAuditLogPermission = RetrieveAuditLogPermission

class RetrieveDataRoomStatusPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___RetrieveDataRoomStatusPermission = RetrieveDataRoomStatusPermission

class UpdateDataRoomStatusPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___UpdateDataRoomStatusPermission = UpdateDataRoomStatusPermission

class RetrievePublishedDatasetsPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___RetrievePublishedDatasetsPermission = RetrievePublishedDatasetsPermission

class DryRunPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___DryRunPermission = DryRunPermission

class GenerateMergeSignaturePermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___GenerateMergeSignaturePermission = GenerateMergeSignaturePermission

class ExecuteDevelopmentComputePermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___ExecuteDevelopmentComputePermission = ExecuteDevelopmentComputePermission

class MergeConfigurationCommitPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___MergeConfigurationCommitPermission = MergeConfigurationCommitPermission
